@@ -1,3 +1,8 @@
+from .iam_pomes import (
+    IamServer, logger_register,
+    login_callback, token_exchange,
+    user_login, user_logout, user_token
+)
 from .jusbr_pomes import (
     jusbr_setup, jusbr_get_token
 )
@@ -12,6 +17,10 @@ from .token_pomes import (
 )
 
 __all__ = [
+    # iam_pomes
+    "IamServer", "logger_register",
+    "login_callback", "token_exchange",
+    "user_login", "user_logout", "user_token",
     # jusbr_pomes
     "jusbr_setup", "jusbr_get_token",
     # keycloak_pomes
