@@ -1,13 +1,14 @@
+import sys
 from cachetools import Cache, FIFOCache
-from datetime import datetime
 from flask import Flask
 from logging import Logger
 from pypomes_core import (
-    APP_PREFIX, TZ_LOCAL, env_get_int, env_get_str
+    APP_PREFIX, env_get_int, env_get_str
 )
 from typing import Any, Final
 
-from .iam_common import IAM_SERVERS, IamServer, _service_token
+from .iam_common import _IAM_SERVERS, IamServer, _iam_lock
+from .iam_pomes import user_token
 
 JUSBR_CLIENT_ID: Final[str] = env_get_str(key=f"{APP_PREFIX}_JUSBR_CLIENT_ID")
 JUSBR_CLIENT_SECRET: Final[str] = env_get_str(key=f"{APP_PREFIX}_JUSBR_CLIENT_SECRET")
@@ -24,6 +25,8 @@ JUSBR_ENDPOINT_TOKEN: Final[str] = env_get_str(key=f"{APP_PREFIX}_JUSBR_ENDPOINT
 
 JUSBR_PUBLIC_KEY_LIFETIME: Final[int] = env_get_int(key=f"{APP_PREFIX}_JUSBR_PUBLIC_KEY_LIFETIME",
                                                     def_value=86400)  # 24 hours
+JUSBR_RECIPIENT_ATTR: Final[str] = env_get_str(key=f"{APP_PREFIX}_JUSBR_RECIPIENT_ATTR",
+                                               def_value="preferred_username")
 JUSBR_URL_AUTH_BASE: Final[str] = env_get_str(key=f"{APP_PREFIX}_JUSBR_URL_AUTH_BASE")
 
 
@@ -32,12 +35,12 @@ def jusbr_setup(flask_app: Flask,
                 client_secret: str = JUSBR_CLIENT_SECRET,
                 client_timeout: int = JUSBR_CLIENT_TIMEOUT,
                 public_key_lifetime: int = JUSBR_PUBLIC_KEY_LIFETIME,
+                recipient_attribute: str = JUSBR_RECIPIENT_ATTR,
                 callback_endpoint: str = JUSBR_ENDPOINT_CALLBACK,
                 token_endpoint: str = JUSBR_ENDPOINT_TOKEN,
                 login_endpoint: str = JUSBR_ENDPOINT_LOGIN,
                 logout_endpoint: str = JUSBR_ENDPOINT_LOGOUT,
-                base_url: str = JUSBR_URL_AUTH_BASE,
-                logger: Logger = None) -> None:
+                base_url: str = JUSBR_URL_AUTH_BASE) -> None:
     """
     Configure the JusBR IAM.
 
@@ -48,29 +51,30 @@ def jusbr_setup(flask_app: Flask,
     :param client_secret: the client's password with JusBR
     :param client_timeout: timeout for login authentication (in seconds,defaults to no timeout)
     :param public_key_lifetime: how long to use JusBR's public key, before refreshing it (in seconds)
+    :param recipient_attribute: attribute in the token's payload holding the token's subject
     :param callback_endpoint: endpoint for the callback from JusBR
-    :param token_endpoint: endpoint for retrieving the JusBR authentication token
-    :param login_endpoint: endpoint for redirecting user to JusBR login page
+    :param token_endpoint: endpoint for retrieving JusBR's authentication token
+    :param login_endpoint: endpoint for redirecting user to JusBR's login page
     :param logout_endpoint: endpoint for terminating user access to JusBR
-    :param base_url: base URL to request the JusBR services
-    :param logger: optional logger
+    :param base_url: base URL to request JusBR services
     """
-    from .iam_pomes import service_login, service_logout, service_callback, service_token
+    from .iam_services import service_login, service_logout, service_callback, service_token
 
     # configure the JusBR registry
     cache: Cache = FIFOCache(maxsize=1048576)
     cache["users"] = {}
-    IAM_SERVERS[IamServer.IAM_JUSRBR] = {
-        "client-id": client_id,
-        "client-secret": client_secret,
-        "client-timeout": client_timeout,
-        "base-url": base_url,
-        "pk-expiration": int(datetime.now(tz=TZ_LOCAL).timestamp()),
-        "pk-lifetime": public_key_lifetime,
-        "cache": cache,
-        "logger": logger,
-        "redirect-uri": None
-    }
+    with _iam_lock:
+        _IAM_SERVERS[IamServer.IAM_JUSRBR] = {
+            "client-id": client_id,
+            "client-secret": client_secret,
+            "client-timeout": client_timeout,
+            "recipient-attr": recipient_attribute,
+            "base-url": base_url,
+            "pk-expiration": sys.maxsize,
+            "pk-lifetime": public_key_lifetime,
+            "cache": cache,
+            "redirect-uri": None
+        }
 
     # establish the endpoints
     if login_endpoint:
@@ -87,7 +91,7 @@ def jusbr_setup(flask_app: Flask,
         flask_app.add_url_rule(rule=callback_endpoint,
                                endpoint="jusbr-callback",
                                view_func=service_callback,
-                               methods=["GET", "POST"])
+                               methods=["GET"])
     if token_endpoint:
         flask_app.add_url_rule(rule=token_endpoint,
                                endpoint="jusbr-token",
@@ -106,9 +110,14 @@ def jusbr_get_token(user_id: str,
     :param logger: optional logger
     :return: the uthentication tokem
     """
+    # declare the return variable
+    result: str
+
     # retrieve the token
     args: dict[str, Any] = {"user-id": user_id}
-    return _service_token(registry=IAM_SERVERS[IamServer.IAM_JUSRBR],
-                          args=args,
-                          errors=errors,
-                          logger=logger)
+    with _iam_lock:
+        result = user_token(iam_server=IamServer.IAM_JUSRBR,
+                            args=args,
+                            errors=errors,
+                            logger=logger)
+    return result
